class Parent:

    def inherited_method(self):
        """Duis lacus metus, euismod ut viverra sit amet, pulvinar sed urna."""
        pass

class Example(Parent):
    """
    Lorem ipsum dolor sit amet, consectetur adipiscing elit.

    Nam justo sem, malesuada ut ultricies ac, bibendum eu neque. Lorem ipsum 
    dolor sit amet, consectetur adipiscing elit. Aenean at tellus ut velit 
    dignissim tincidunt. Curabitur euismod laoreet orci semper dignissim. 
    Suspendisse potenti. 
    """

    def __init__(self):
        """Vivamus sed enim quis dui pulvinar pharetra."""
        pass

    def documented_method(self):
        """
        Duis condimentum ultricies ipsum, sed ornare leo vestibulum vitae. 

        Sed ut justo massa, varius molestie diam. Sed lacus quam, tempor in 
        dictum sed, posuere et diam. Maecenas tincidunt enim elementum turpis 
        blandit tempus. Nam lectus justo, adipiscing vitae ultricies egestas, 
        porta nec diam. Aenean ac neque tortor. 
        """
        pass

    def undocumented_method(self):
        pass

    def _private_method(self):
        """Cras tempus lacus nec leo ultrices suscipit."""
        pass



import sys, os
import autoclasstoc

sys.path.append(os.path.dirname(__file__))

project = u'autoclasstoc'
copyright = u'2020, Kale Kundert'
version = autoclasstoc.__version__
release = autoclasstoc.__version__

templates_path = ['_templates']
exclude_patterns = ['_build']
html_static_path = ['_static']
source_suffix = '.rst'
master_doc = 'index'
default_role = 'any'

extensions = [
        '_ext.example',
        '_ext.show_nodes',
        'autoclasstoc',
        'sphinx.ext.autodoc',
        'sphinx.ext.autosectionlabel',
        'sphinx.ext.autosummary',
        'sphinx.ext.viewcode',
        'sphinx.ext.intersphinx',
        'sphinx.ext.napoleon',
        'sphinx_rtd_theme',
]

intersphinx_mapping = {
        'python': ('https://docs.python.org/3', None),
        'sphinx': ('https://www.sphinx-doc.org/', None),
}
autodoc_default_options = {
        'exclude-members': '__weakref__,__dict__,__module__',
}
autosummary_generate = True
pygments_style = 'sphinx'
html_theme = 'sphinx_rtd_theme'

def setup(app):
    app.add_css_file('css/autoclasstoc.css')




#!/usr/bin/env python3

"""\
The :mod:`autoclasstoc` module defines two new *docutils* nodes, which make it 
possible to create collapsible content in HTML.
"""

from docutils.nodes import General, Element, TextElement


class details(General, Element):
    """
    A node that can be expanded or collapsed by the user.

    This is rendered as a ``<details>`` element in HTML.  It is not currently 
    compatible with non-HTML output formats.
    """

    def __init__(self, open_by_default=False):
        super().__init__()
        self['open'] = open_by_default

    def visit_html(visitor, node):
        atts = {
            'class': ' '.join(node['classes'])
        }
        parts = ['details'] + [
            f'{k}="{v}"'
            for k, v in atts.items()
            if v
        ]
        if node['open']:
            parts += 'open'

        visitor.body.append(f"<{' '.join(parts)}>")

    def depart_html(visitor, node):
        visitor.body.append('</details>')

    html = visit_html, depart_html


class details_summary(General, TextElement):
    """
    The summary text to display when a `details` node is collapsed.

    This is rendered as a ``<summary>`` element in HTML.  It is not currently 
    compatible with non-HTML output formats.
    """

    def visit_html(visitor, node):
        visitor.body.append('<summary>')

    def depart_html(visitor, node):
        visitor.body.append('</summary>')

    html = visit_html, depart_html


def setup(app):
    """
    Configure Sphinx to use the `details` and `details_summary` nodes.
    """
    app.add_node(
        details,
        html=details.html,
    )
    app.add_node(
        details_summary,
        html=details_summary.html,
    )



#!/usr/bin/env python3

from docutils import nodes
from docutils.parsers.rst import Directive
from sphinx.util.docutils import ReferenceRole
from autoclasstoc import utils

class Example(Directive):
    has_content = True

    def run(self):
        wrapper = nodes.paragraph()
        wrapper += utils.nodes_from_rst(self.state, [
                '.. code-block:: rst',
                '',
                *['    ' + x for x in self.content],
        ])
        wrapper += utils.nodes_from_rst(self.state, self.content)
        return wrapper.children

class SphinxExtRole(ReferenceRole):

    def run(self):
        title = f'sphinx.ext.{self.text}'
        url = f'https://www.sphinx-doc.org/en/master/usage/extensions/{self.text}.html'
        a = nodes.reference('', '', internal=False, refuri=url)
        a += nodes.literal(title, title)
        return [a], []

def setup(app):
    app.add_directive('example', Example)
    app.add_role('ext', SphinxExtRole())
    app.add_object_type(
            'confval',
            'confval',
            objname='configuration value',
            indextemplate='pair: %s; configuration value',
    )






#!/usr/bin/env python3

import pytest
import autoclasstoc

class DummySection1(autoclasstoc.Section):
    key = 'dummy-section-1'
    title = "Dummy Section 1:"

class DummySection2(autoclasstoc.Section):
    key = 'dummy-section-2'
    title = "Dummy Section 2:"

@pytest.mark.parametrize(
        'given, exclude, expected', [(
            [],
            [],
            [],
        ), (
            [DummySection1],
            [],
            [DummySection1],
        ), (
            ['dummy-section-1'],
            [],
            [DummySection1],
        ), (
            [DummySection1, DummySection2],
            [],
            [DummySection1, DummySection2],
        ), (
            [DummySection1, 'dummy-section-2'],
            [],
            [DummySection1, DummySection2],
        ), (
            ['dummy-section-1', DummySection2],
            [],
            [DummySection1, DummySection2],
        ), (
            ['dummy-section-1', 'dummy-section-2'],
            [],
            [DummySection1, DummySection2],
        ), (
            [DummySection1, DummySection2],
            [DummySection1],
            [DummySection2],
        ), (
            [DummySection1, DummySection2],
            ['dummy-section-1'],
            [DummySection2],
        ), (
            [DummySection1, DummySection2],
            [DummySection2],
            [DummySection1],
        ), (
            [DummySection1, DummySection2],
            ['dummy-section-2'],
            [DummySection1],
        ), (
            [DummySection1],
            [DummySection2],
            [DummySection1],
)])
def test_pick_sections(given, exclude, expected):
    assert autoclasstoc.utils.pick_sections(given, exclude) == expected

@pytest.mark.parametrize(
        'given, exclude, err', [
            (['unknown-section'], [], "unknown-section"),
            ([], ['unknown-section'], "unknown-section"),
        ]
)
def test_pick_sections_err(given, exclude, err):
    with pytest.raises(autoclasstoc.ConfigError, match=err):
        autoclasstoc.utils.pick_sections(given, exclude)



#!/usr/bin/env python3

"""\
Add a succinct TOC to auto-documented classes.
"""

__version__ = '1.4.0'

from .errors import *
from . import utils, nodes
from .sections import *
from .plugin import *

# Make this package appear flat to external tools (e.g. sphinx):
from inspect import isfunction, isclass

for obj in locals().copy().values():
    if isfunction(obj) or isclass(obj):
        obj.__module__ = 'autoclasstoc'

del inspect, isfunction, isclass, obj



#!/usr/bin/env python3

from sphinx.util.docutils import SphinxDirective
from . import utils, __version__, ConfigError


class AutoClassToc(SphinxDirective):
    """
    Generate a succinct TOC for automatically documented classes.

    This class implements the :rst:dir:`autoclasstoc` directive.  More 
    specifically, it implements the `run` function as expected by docutils.  
    However, most of the actual logic is delegated to other classes and 
    functions.
    """
    optional_arguments = 1
    option_spec = {
        'sections': utils.comma_separated_list,
        'exclude-sections': utils.comma_separated_list,
    }

    def run(self):
        """
        Create the nodes that will represent the class TOC.
        """
        try:
            qual_name = self.arguments[0] if self.arguments else None
            mod_name, cls_name = utils.pick_class(qual_name, self.env)
            mod, cls = utils.load_class(mod_name, cls_name)
            sections = utils.pick_sections(
                self.options.get('sections') or
                self.config.autoclasstoc_sections,
                exclude=self.options.get('exclude-sections'),
            )
            return utils.make_toc(self.state, cls, sections)

        except ConfigError as err:
            raise self.error(str(err))


def load_static_assets(app, config):
    """
    Add some rules for the spacing around <details> elements in class TOCs.
    """
    from pathlib import Path
    static_dir = Path(__file__).parent / '_static'
    static_dir = str(static_dir.resolve())

    if static_dir not in config.html_static_path:
        config.html_static_path.append(static_dir)

    app.add_css_file('autoclasstoc.css')


def setup(app):
    from . import nodes
    nodes.setup(app)

    default_sections = [
        'public-attrs',
        'public-methods',
        'private-attrs',
        'private-methods'
    ]

    app.add_config_value('autoclasstoc_sections', default_sections, 'env')
    app.add_directive('autoclasstoc', AutoClassToc)
    app.connect('config-inited', load_static_assets)

    return {
        'version': __version__,
        'parallel_read_safe': True,
    }



#!/usr/bin/env python3

from docutils import nodes as _nodes
from docutils.statemachine import StringList, string2lines
from importlib import import_module
from .errors import ConfigError


def pick_class(qual_name, env):
    """
    Figure out which class to make the TOC for.

    We can either be given this information as an argument, or we can try to 
    figure it out from the context (e.g. the :rst:dir:`autoclass` or 
    :rst:dir:`py:class` currently being processed).

    Arguments:
        qual_name (str): The name of the class to pick, or None if the class 
        should be inferred from the environment.
        env (sphinx.environment.BuildEnvironment): This object is available as 
        :attr:`self.env` from :class:`~sphinx.util.docutils.SphinxDirective` 
        subclasses.
    """
    if qual_name:
        mod_name, cls_name = qual_name.rsplit('.', 1)
    else:
        cls_name = \
            env.temp_data.get('autodoc:class') or \
            env.ref_context.get('py:class')
        mod_name = \
            env.temp_data.get('autodoc:module') or \
            env.ref_context.get('py:module')

        if not cls_name:
            raise ConfigError("no class name")
        if not mod_name:
            raise ConfigError("no module name")

    return mod_name, cls_name


def load_class(mod_name, cls_name):
    """
    Import the given class from the given module.
    """
    mod = import_module(mod_name)
    cls = getattr(mod, cls_name)
    return mod, cls


def pick_sections(sections, exclude=None):
    """
    Determine which sections to include in the class TOC.

    The return value will be a list in the same order as *sections*, but with 
    any sections from *exclude* removed.  Both arguments can specify sections 
    using string names (e.g. "public-methods") or un-instantiated `Section` 
    classes.  All names will be converted to classes in the return value.
    """

    def _section_from_anything(x):
        from .sections import Section, SECTIONS

        if isinstance(x, str):
            try:
                return SECTIONS[x]
            except KeyError:
                raise ConfigError(f"no autoclasstoc section with key {x!r}")

        if issubclass(x, Section):
            return x

        raise ConfigError(f"cannot interpret {x!r} as a section")

    sections = [
        _section_from_anything(x)
        for x in sections
    ]
    exclude = {
        _section_from_anything(x)
        for x in exclude or []
    }
    return [
        x
        for x in sections
        if x not in exclude
    ]


def make_toc(state, cls, sections):
    """
    Create the class TOC.
    """
    n = []
    for section_cls in sections:
        section = section_cls(state, cls)
        section.check()
        n += section.format()

    n.append(_nodes.transition())
    return n


def make_container():
    """
    Make a container node to identify elements associated with the 
    :rst:dir:`autoclasstoc` directive.
    """
    return _nodes.container(classes=['autoclasstoc'])


def make_rubric(title):
    """
    Make an informal header.
    """
    return _nodes.rubric(title, title)


def make_inherited_details(state, parent, open_by_default=False):
    """
    Make a collapsible node to contain information about inherited attributes.
    """
    from .nodes import details, details_summary
    s = details_summary()
    s += strip_p(nodes_from_rst(state,
                                f"Inherited from    : py: class:`{parent.__qualname__}`"))

    d = details(open_by_default)
    d += s
    return d


def make_links(state, attrs):
    """
    Make links to the given class attributes.

    More specifically, the links are made using the :rst:dir:`autosummary` 
    directive.
    """
    assert attrs
    return nodes_from_rst(state, [
        '.. autosummary::',
        '',
        *[f'    {x}' for x in attrs],
    ])


def find_inherited_attrs(cls):
    """
    Return a dictionary mapping parent classes to the attributes inherited from 
    those classes.
    """
    return {
        base: base.__dict__
        for base in cls.__mro__
        if base not in (cls, object)
    }


def filter_attrs(attrs, predicate):
    """
    Remove attributes for which the given predicate function returns False.
    """
    from inspect import getdoc
    from sphinx.util.docstrings import extract_metadata

    return {
        k: v
        for k, v in attrs.items()
        if predicate(k, v, extract_metadata(getdoc(v)))
    }


def nodes_from_rst(state, rst):
    """
    Create nodes from the given restructured text.

    The *rst* argument can either be any of the following types:

    - string, with any number of lines
    - list of strings
    - StringList (the type used by docutils to represent lines of restructured 
      text)
    - node
    """
    if isinstance(rst, str):
        rst = string2lines(rst)
    if isinstance(rst, list):
        rst = StringList(rst)
    if isinstance(rst, _nodes.Node):
        return rst

    wrapper = _nodes.paragraph()
    state.nested_parse(rst, 0, wrapper)
    return wrapper.children


def strip_p(nodes):
    """
    Remove any top-level paragraph nodes.

    Parsing a simple string like "Hello world" with `nodes_from_rst` will 
    return text wrapped in a paragraph.  If this paragraph is not desired (e.g.  
    because it messes with formatting), this function can be used to get rid of 
    it.
    """
    while len(nodes) == 1 and isinstance(nodes[0], _nodes.paragraph):
        nodes = nodes[0].children
    return nodes


def comma_separated_list(x):
    """
    Parse a restructured text option as a comma-separated list of strings.
    """
    return x.split(',')



#!/usr/bin/env python3

from docutils import nodes
from docutils.parsers.rst import Directive

class ShowNodesDirective(Directive):
    """
    Pretty print all the nodes in the restructured-text contained in this 
    directive.  This is only meant to be a tool for developing and debugging 
    new Sphinx extensions.

    Example:

        .. show-nodes::

            .. figure:: path/to/image.png

                Caption...
    """
    has_content = True

    def run(self):
        wrapper = nodes.paragraph()
        self.state.nested_parse(self.content, self.content_offset, wrapper)

        for node in wrapper.children:
            print(node.pformat())

        return wrapper.children


def setup(app):
    app.add_directive('shownodes', ShowNodesDirective)



#!/usr/bin/env python3

import inspect
from docutils import nodes as _nodes
from more_itertools import always_iterable
from . import utils
import re

SECTIONS = {}


class Section:
    """
    Format a specific section in a class TOC, e.g. "Public Methods".

    The purpose of this class is to make it easy to customize the sections that 
    make up the class TOC.  For example, you might want an "Event Handler" 
    section that includes any method that starts with "on\\_".  Or you might 
    want to format the links in a table with multiple columns, to save more 
    space.  

    These kinds of things can be accomplished by subclassing `Section` and 
    overwriting the relevant methods.  Almost every method is meant to be 
    overridden by subclasses, but most subclasses will only need to override 
    `key`, `title`, and `predicate`.  `key` and `title` have no default value, 
    and must be overridden in each subclass.  `predicate` determines which 
    attributes are included in the section, which is the primary purpose of 
    most custom sections.
    """

    key = None
    """
    A string that can be used to refer to this section, e.g. in ``conf.py`` 
    settings such as :confval:`autoclasstoc_sections` and 
    :rst:dir:`autoclasstoc` options such as ``:sections:`` and 
    ``:exclude-sections:``.
    """

    title = None
    """
    The text that will be used to label this section.

    It's conventional to end the title with a trailing colon.
    """

    include_inherited = True
    """
    Whether or not to include inherited attributes in this section.
    """

    exclude_pattern = None
    """
    A regular expression (or list of regular expressions) matching attribute 
    names that should be excluded from this section.
    """

    def __init__(self, state, cls):
        """
        Create a section for a specific class.

        Arguments:
            state (docutils.parsers.rst.states.RSTState): The state object 
                associated with the :rst:dir:`autoclasstoc` directive.  This 
                can be used to evaluate restructured text markup using 
                `nodes_from_rst()`.  
            cls (type): The class to make the TOC section for.
        """
        self.state = state
        self.cls = cls

    def __init_subclass__(cls):
        """
        Keep track of any `Section` subclasses that are defined.
        """
        SECTIONS[cls.key] = cls

    def check(self):
        """
        Raise `ConfigError` if the section has not been configured correctly, 
        e.g. if it doesn't have a title specified.
        """
        if not self.key:
            raise ConfigError(
                f"no key specified for {self.__class__.__name__!r}")
        if not self.title:
            raise ConfigError(
                f"no title specified for {self.__class__.__name__!r}")

    def format(self):
        """
        Return a list of *docutils* nodes that will compose the section.

        The default implementation of this method creates and populates 
        :rst:dir:`autosummary` directives for the class in question and all of 
        its superclasses.  Almost all of 
        """
        assert self.state
        assert self.cls

        attrs = self._filter_attrs(self._find_attrs())
        inherited_attrs = {
            parent: self._filter_attrs(attrs)
            for parent, attrs in self._find_inherited_attrs().items()
        }

        if not attrs and not any(inherited_attrs.values()):
            return []

        wrapper = self._make_container()
        wrapper += self._make_rubric()

        if attrs:
            wrapper += self._make_links(attrs)

        if not self.include_inherited:
            return [wrapper]

        for parent, attrs in inherited_attrs.items():
            if not attrs:
                continue

            d = self._make_inherited_details(parent)
            d += self._make_links(attrs)
            wrapper += d

        return [wrapper]

    def predicate(self, name, attr, meta):
        """
        Return true if the given attribute should be included in this section.

        Arguments:
            name (str): The name of the attribute.  In most cases, this is 
                identical to :attr:`attr.__name__`.
            attr (object): The attribute object itself.
            meta (dict): Any `:meta:`__ fields present in the attribute's 
                docstring, as parsed by 
                :func:`sphinx.util.docstrings.extract_metadata()`. 

        __ https://www.sphinx-doc.org/en/master/usage/restructuredtext/domains.html#info-field-lists

        See Also:
            `is_method`
            `is_data_attr`
            `is_public`
            `is_private`
            `is_special`
        """
        return not does_match(name, self.exclude_pattern)

    def _make_container(self):
        """
        Create the container node that will contain the entire section.

        This method is meant to be overridden in subclasses.  The primary 
        purpose of the container node is to belong to a CSS class that can then 
        be used to identify HTML elements associated with 
        :rst:dir:`autoclasstoc`.
        """
        return utils.make_container()

    def _make_rubric(self):
        """
        Create the section header node.

        This method is meant to be overridden in subclasses.
        """
        return utils.make_rubric(self.title)

    def _make_links(self, attrs):
        """
        Make a link to the full documentation for each attribute.

        This method is meant to be overridden in subclasses.  The default 
        implementation creates the links using an :rst:dir:`autosummary` 
        directive.

        Arguments:
            attrs (dict): A dictionary of attributes, in the same format as 
                ``__dict__``.
        """
        return utils.make_links(self.state, attrs)

    def _make_inherited_details(self, parent):
        """
        Make a collapsible node to contain links to inherited attributes.

        This method is meant to be overridden in subclasses.  The default 
        implementation returns a `details` node, which is rendered in HTML as a 
        ``<details>`` element.
        """
        return utils.make_inherited_details(self.state, parent)

    def _filter_attrs(self, attrs):
        """
        Return only those attributes that match the predicate.

        Arguments:
            attrs (dict): A dictionary of attributes, in the same format as 
                ``__dict__``.

        Return:
            A dictionary in the same format as *attrs*.
        """
        return utils.filter_attrs(attrs, self.predicate)

    def _find_attrs(self):
        """
        Return all attributes associated with this class.

        These attributes will subsequently be filtered to remove any that 
        aren't relevant to this section, so there is no need to do any 
        filtering here.  The return value should be a name-to-attribute 
        dictionary in the same format as :attr:`__dict__`.
        """
        return self.cls.__dict__

    def _find_inherited_attrs(self):
        """
        Find attributes that this class has inherited from other classes.

        These attributes will subsequently be filtered to remove any that 
        aren't relevant to this section, so there is no need to do any 
        filtering here.  The return value should be a dictionary mapping parent 
        class types to :attr:`__dict__` style dictionaries.
        """
        return utils.find_inherited_attrs(self.cls)


class PublicMethods(Section):
    """
    Include a "Public Methods" section in the class TOC.
    """
    key = 'public-methods'
    title = "Public Methods:"

    def predicate(self, name, attr, meta):
        return (
            super().predicate(name, attr, meta) and
            is_method(name, attr) and
            is_public(name)
        )


class PublicMethodsWithoutDunders(PublicMethods):
    """
    Include a "Public Methods" section in the class TOC.
    """
    key = 'public-methods-without-dunders'
    exclude_pattern = '__'


class PrivateMethods(Section):
    """
    Include a "Private Methods" section in the class TOC.
    """
    key = 'private-methods'
    title = "Private Methods:"

    def predicate(self, name, attr, meta):
        return (
            super().predicate(name, attr, meta) and
            is_method(name, attr) and
            is_private(name)
        )


class PublicDataAttrs(Section):
    """
    Include a "Public Data Attributes" section in the class TOC.

    Note that only data attributes defined at the class level will be included 
    in the TOC.  Data attributes defined in :meth:`__init__` (for example) will 
    not be found.
    """
    key = 'public-attrs'
    title = "Public Data Attributes:"

    def predicate(self, name, attr, meta):
        return (
            super().predicate(name, attr, meta) and
            is_data_attr(name, attr) and
            is_public(name)
        )


class PrivateDataAttrs(Section):
    """
    Include a "Private Data Attributes" section in the class TOC.

    Note that only data attributes defined at the class level will be included 
    in the TOC.  Data attributes defined in :meth:`__init__` (for example) will 
    not be found.
    """
    key = 'private-attrs'
    title = "Private Data Attributes:"

    def predicate(self, name, attr, meta):
        return (
            super().predicate(name, attr, meta) and
            is_data_attr(name, attr) and
            is_private(name)
        )


class InnerClasses(Section):
    """
    Include an "Inner Classes" section in the class TOC.
    """
    key = 'inner-classes'
    title = "Inner Classes:"

    def predicate(self, name, attr, meta):
        return (
            super().predicate(name, attr, meta) and
            inspect.isclass(attr)
        )


def is_method(name, attr):
    """
    Return true if the given attribute is a method or property.
    """
    return any([
        inspect.isfunction(attr),
        inspect.ismethoddescriptor(attr),
    ])


def is_data_attr(name, attr, exclude_special=True):
    """
    Return true if the given attribute is a data attribute, e.g. not a method 
    or an inner class.  Many data attributes are properties.

    By default, attributes with double-underscore names (e.g. :attr:`__dict__`) 
    are not considered data attributes.  Unlike special methods, these "special 
    attributes" are very rarely relevant to users of a class.  This behavior 
    can be disabled by toggling the *exclude_special* argument.
    """
    if exclude_special and is_special(name):
        return False

    return all([
        not inspect.isclass(attr),
        not inspect.isfunction(attr),
        not inspect.ismethoddescriptor(attr),
    ])


def is_public(name):
    """
    Return true if the given name is public.

    Specifically, a name is public if it either doesn't start with an 
    underscore, or if it starts and ends with two underscores (i.e. a "special"
    method).
    """
    return not name.startswith('_') or is_special(name)


def is_private(name):
    """
    Return true if the given name is private.

    A name is private if it starts with an underscore, but does not start and 
    end with two underscores (i.e. not a special method).
    """
    return not is_public(name)


def is_special(name):
    """
    Return True if the name starts and ends with a double-underscore.

    Such names typically have special meaning to Python, e.g. :meth:`__init__`.
    """
    return name.startswith('__') and name.endswith('__')


def does_match(name, pattern, **kwargs):
    """
    Return true if the name matches the given pattern.

    Under the hood, `re.match` is used to find the pattern.  This means that 
    the match must start at the beginning of the name.  If you want to match an 
    internal pattern, the pattern must start with ``.*``.
    """
    return pattern and any(
            re.match(p, name, **kwargs)
            for p in always_iterable(pattern)
    )




#!/usr/bin/env python3

class AutoClassTocError(Exception):
    pass

class ConfigError(AutoClassTocError):
    """
    Indicate an configuration error affecting :rst:dir:`autoclasstoc`.
    """
    pass





#!/usr/bin/env python3

import pytest
import autoclasstoc
import parametrize_from_file as pff

with_autoclasstoc = pff.Namespace('from autoclasstoc import *')

class MockObj:
    data_attr = None
    _private_data_attr = None

    class InnerClass:
        pass
    
    def __init__(self):
        pass

    def method(self):
        pass

    def _private_method(self):
        pass

class EmptyObj:
    pass

@pytest.mark.parametrize(
        'name, pattern, expected', [
            ('', 'ab', False),
            ('ab', 'ab', True),
            ('ba', 'ab', False),

            ('', ['ab', 'cd'], False),
            ('ab', ['ab', 'cd'], True),
            ('ba', ['ab', 'cd'], False),
            ('cd', ['ab', 'cd'], True),
            ('dc', ['ab', 'cd'], False),
            ('abcd', ['ab', 'cd'], True),
            ('abdc', ['ab', 'cd'], True),
            ('bacd', ['ab', 'cd'], False),
            ('bacd', ['ab', '.*cd'], True),
            ('badc', ['ab', 'cd'], False),

            # Test some real-world patterns.
            ('__init__', '__', True),
            ('__len__', '__', True),
            ('foo', '__', False),
            ('_bar', '__', False),

            ('__init__', ['__', 'on'], True),
            ('__len__', ['__', 'on'], True),
            ('foo', ['__', 'on'], False),
            ('_bar', ['__', 'on'], False),
            ('on_off', ['__', 'on'], True),
            ('off_on', ['__', 'on'], False),
            ('off_on', ['__', '.*on'], True),
    ]
)
def test_does_match(name, pattern, expected):
    assert autoclasstoc.does_match(name, pattern) == expected


@pff.parametrize
def test_section_predicate(section, expected):
    # For the sake of making the expected values easy to predict, remove any 
    # attributes that python automatically adds to new classes.
    attrs = {
            k: v
            for k, v in MockObj.__dict__.items()
            if k not in EmptyObj.__dict__
    }
    section_cls = with_autoclasstoc.exec(section, get='MockSection')
    section = section_cls('state', 'cls')
    hits = autoclasstoc.utils.filter_attrs(attrs, section.predicate)
    assert set(hits) == set(expected)

